@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,6 +26,8 @@
 #include <optional>
 #include <algorithm>
 #include <functional>
+#include <iostream>
+#include <fstream>
 #include <cstdint>
 #include <cstdio>
 #include <cmath>
@@ -80,21 +82,22 @@ static gboolean ucl_reply = FALSE;
 static gboolean extended_urls = FALSE;
 static gboolean mime_output = FALSE;
 static gboolean empty_input = FALSE;
-static gboolean compressed = FALSE;
+static gboolean compressed = TRUE;
 static gboolean profile = FALSE;
 static gboolean skip_images = FALSE;
 static gboolean skip_attachments = FALSE;
 static const char *pubkey = nullptr;
 static const char *user_agent = "rspamc";
+static const char *files_list = nullptr;
 
 std::vector<GPid> children;
 static GPatternSpec **exclude_compiled = nullptr;
 static struct rspamd_http_context *http_ctx;
 
-static gint retcode = EXIT_SUCCESS;
+static int retcode = EXIT_SUCCESS;
 
-static gboolean rspamc_password_callback(const gchar *option_name,
-										 const gchar *value,
+static gboolean rspamc_password_callback(const char *option_name,
+										 const char *value,
 										 gpointer data,
 										 GError **error);
 
@@ -176,6 +179,8 @@ static GOptionEntry entries[] =
 		 "Skip attachments when learning/unlearning fuzzy", nullptr},
 		{"user-agent", 'U', 0, G_OPTION_ARG_STRING, &user_agent,
 		 "Use specific User-Agent instead of \"rspamc\"", nullptr},
+		{"files-list", '\0', 0, G_OPTION_ARG_FILENAME, &files_list,
+		 "Read one or more newline separated filenames to scan from file", nullptr},
 		{nullptr, 0, 0, G_OPTION_ARG_NONE, nullptr, nullptr, nullptr}};
 
 static void rspamc_symbols_output(FILE *out, ucl_object_t *obj);
@@ -362,6 +367,41 @@ struct fmt::formatter<rspamd_action_type> : fmt::formatter<string_view> {
 	}
 };
 
+template<typename... T>
+static inline void rspamc_print(std::FILE *f, fmt::format_string<T...> fmt, T &&...args)
+{
+	static auto try_print_exception = true;
+	auto wanna_die = false;
+
+	try {
+		fmt::print(f, fmt, std::forward<T>(args)...);
+	} catch (const fmt::format_error &err) {
+		if (try_print_exception) {
+			if (fprintf(stderr, "Format error: %s\n", err.what()) < 0) {
+				try_print_exception = false;
+			}
+		}
+	} catch (std::system_error &err) {
+		wanna_die = true;
+		if (try_print_exception) {
+			if (fprintf(stderr, "System error: %s\n", err.what()) < 0) {
+				try_print_exception = false;
+			}
+		}
+	} catch (...) {
+		wanna_die = true;
+		if (try_print_exception) {
+			if (fprintf(stderr, "Unknown format error\n") < 0) {
+				try_print_exception = false;
+			}
+		}
+	}
+
+	if (wanna_die) {
+		exit(EXIT_FAILURE);
+	}
+}
+
 using sort_lambda = std::function<int(const ucl_object_t *, const ucl_object_t *)>;
 static const auto sort_map = frozen::make_unordered_map<frozen::string, sort_lambda>({
 	{"name", [](const ucl_object_t *o1, const ucl_object_t *o2) -> int {
@@ -461,8 +501,8 @@ auto sort_ucl_container_with_default(T &cont, const char *default_sort,
 
 
 static gboolean
-rspamc_password_callback(const gchar *option_name,
-						 const gchar *value,
+rspamc_password_callback(const char *option_name,
+						 const char *value,
 						 gpointer data,
 						 GError **error)
 {
@@ -507,7 +547,7 @@ rspamc_password_callback(const gchar *option_name,
 		processed_passwd.resize(plen, '\0');
 		plen = rspamd_read_passphrase(processed_passwd.data(), plen, 0, nullptr);
 		if (plen == 0) {
-			fmt::print(stderr, "Invalid password\n");
+			rspamc_print(stderr, "Invalid password\n");
 			exit(EXIT_FAILURE);
 		}
 		processed_passwd.resize(plen);
@@ -523,7 +563,7 @@ rspamc_password_callback(const gchar *option_name,
  * Parse command line
  */
 static void
-read_cmd_line(gint *argc, gchar ***argv)
+read_cmd_line(int *argc, char ***argv)
 {
 	GError *error = nullptr;
 	GOptionContext *context;
@@ -536,7 +576,7 @@ read_cmd_line(gint *argc, gchar ***argv)
 
 	/* Parse options */
 	if (!g_option_context_parse(context, argc, argv, &error)) {
-		fmt::print(stderr, "option parsing failed: {}\n", error->message);
+		rspamc_print(stderr, "option parsing failed: {}\n", error->message);
 		g_option_context_free(context);
 		exit(EXIT_FAILURE);
 	}
@@ -637,9 +677,9 @@ check_rspamc_command(const char *cmd) -> std::optional<rspamc_command>
 static void
 print_commands_list()
 {
-	guint cmd_len = 0;
+	unsigned int cmd_len = 0;
 
-	fmt::print(stdout, "Rspamc commands summary:\n");
+	rspamc_print(stdout, "Rspamc commands summary:\n");
 
 	for (const auto &cmd: rspamc_commands) {
 		auto clen = strlen(cmd.name);
@@ -650,19 +690,19 @@ print_commands_list()
 	}
 
 	for (const auto &cmd: rspamc_commands) {
-		fmt::print(stdout,
-				   "  {:>{}} ({:7}{:1})\t{}\n",
-				   cmd.name,
-				   cmd_len,
-				   cmd.is_controller ? "control" : "normal",
-				   cmd.is_privileged ? "*" : "",
-				   cmd.description);
+		rspamc_print(stdout,
+					 "  {:>{}} ({:7}{:1})\t{}\n",
+					 cmd.name,
+					 cmd_len,
+					 cmd.is_controller ? "control" : "normal",
+					 cmd.is_privileged ? "*" : "",
+					 cmd.description);
 	}
 
-	fmt::print(stdout,
-			   "\n* is for privileged commands that may need password (see -P option)\n");
-	fmt::print(stdout,
-			   "control commands use port 11334 while normal use 11333 by default (see -h option)\n");
+	rspamc_print(stdout,
+				 "\n* is for privileged commands that may need password (see -P option)\n");
+	rspamc_print(stdout,
+				 "control commands use port 11334 while normal use 11333 by default (see -h option)\n");
 }
 
 static void
@@ -707,9 +747,9 @@ add_options(GQueue *opts)
 				freeaddrinfo(res);
 			}
 			else {
-				fmt::print(stderr, "address resolution for {} failed: {}\n",
-						   ip,
-						   gai_strerror(r));
+				rspamc_print(stderr, "address resolution for {} failed: {}\n",
+							 ip,
+							 gai_strerror(r));
 			}
 		}
 		else {
@@ -847,9 +887,9 @@ rspamc_print_indented_line(FILE *out, std::string_view line) -> void
 			}
 		}
 		if (indent && pos) {
-			fmt::print(out, "{:>{}}", " ", indent);
+			rspamc_print(out, "{:>{}}", " ", indent);
 		}
-		fmt::print(out, "{}\n", s);
+		rspamc_print(out, "{}\n", s);
 		pos = line.find_first_not_of(whitespace, pos + s.size());//skip leading whitespace
 	}
 }
@@ -904,31 +944,31 @@ rspamc_symbol_output(FILE *out, const ucl_object_t *obj)
 {
 	auto first = true;
 
-	fmt::print(out, "Symbol: {} ", ucl_object_key(obj));
+	rspamc_print(out, "Symbol: {} ", ucl_object_key(obj));
 	const auto *val = ucl_object_lookup(obj, "score");
 
 	if (val != nullptr) {
-		fmt::print(out, "({:.2f})", ucl_object_todouble(val));
+		rspamc_print(out, "({:.2f})", ucl_object_todouble(val));
 	}
 	val = ucl_object_lookup(obj, "options");
 	if (val != nullptr && ucl_object_type(val) == UCL_ARRAY) {
 		ucl_object_iter_t it = nullptr;
 		const ucl_object_t *cur;
 
-		fmt::print(out, "[");
+		rspamc_print(out, "[");
 
 		while ((cur = ucl_object_iterate(val, &it, true)) != nullptr) {
 			if (first) {
-				fmt::print(out, "{}", ucl_object_tostring(cur));
+				rspamc_print(out, "{}", ucl_object_tostring(cur));
 				first = false;
 			}
 			else {
-				fmt::print(out, ", {}", ucl_object_tostring(cur));
+				rspamc_print(out, ", {}", ucl_object_tostring(cur));
 			}
 		}
-		fmt::print(out, "]");
+		rspamc_print(out, "]");
 	}
-	fmt::print(out, "\n");
+	rspamc_print(out, "\n");
 }
 
 static void
@@ -942,16 +982,16 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 		auto *elt = ucl_object_lookup(obj, ucl_name);
 		if (elt) {
 			if (humanreport) {
-				fmt::print(out, ",{}={}", output_message, emphasis_argument(ucl_object_tostring(elt)));
+				rspamc_print(out, ",{}={}", output_message, emphasis_argument(ucl_object_tostring(elt)));
 			}
 			else {
-				fmt::print(out, "{}: {}\n", output_message, emphasis_argument(ucl_object_tostring(elt)));
+				rspamc_print(out, "{}: {}\n", output_message, emphasis_argument(ucl_object_tostring(elt)));
 			}
 		}
 	};
 
 	if (!humanreport) {
-		fmt::print(out, "[Metric: default]\n");
+		rspamc_print(out, "[Metric: default]\n");
 	}
 
 	const auto *elt = ucl_object_lookup(obj, "required_score");
@@ -987,12 +1027,12 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 
 
 	if (humanreport) {
-		fmt::print(out,
-				   "{}/{}/{}/{}",
-				   emphasis_argument(score, 2),
-				   emphasis_argument(greylist_score, 2),
-				   emphasis_argument(addheader_score, 2),
-				   emphasis_argument(required_score, 2));
+		rspamc_print(out,
+					 "{}/{}/{}/{}",
+					 emphasis_argument(score, 2),
+					 emphasis_argument(greylist_score, 2),
+					 emphasis_argument(addheader_score, 2),
+					 emphasis_argument(required_score, 2));
 	}
 
 	elt = ucl_object_lookup(obj, "action");
@@ -1002,7 +1042,7 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 		if (act.has_value()) {
 			if (!tty) {
 				if (humanreport) {
-					fmt::print(out, ",action={}:{}", act.value(), ucl_object_tostring(elt));
+					rspamc_print(out, ",action={}:{}", act.value(), ucl_object_tostring(elt));
 				}
 				else {
 					print_protocol_string("action", "Action");
@@ -1013,40 +1053,40 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 				std::string colorized_action;
 				switch (act.value()) {
 				case METRIC_ACTION_REJECT:
-					colorized_action = fmt::format(fmt::fg(fmt::color::red), "reject");
+					colorized_action = fmt::format(fmt::fg(fmt::color::red), "{}", "reject");
 					break;
 				case METRIC_ACTION_NOACTION:
-					colorized_action = fmt::format(fmt::fg(fmt::color::green), "no action");
+					colorized_action = fmt::format(fmt::fg(fmt::color::green), "{}", "no action");
 					break;
 				case METRIC_ACTION_ADD_HEADER:
 				case METRIC_ACTION_REWRITE_SUBJECT:
-					colorized_action = fmt::format(fmt::fg(fmt::color::orange), ucl_object_tostring(elt));
+					colorized_action = fmt::format(fmt::fg(fmt::color::orange), "{}", ucl_object_tostring(elt));
 					break;
 				case METRIC_ACTION_GREYLIST:
 				case METRIC_ACTION_SOFT_REJECT:
-					colorized_action = fmt::format(fmt::fg(fmt::color::gray), ucl_object_tostring(elt));
+					colorized_action = fmt::format(fmt::fg(fmt::color::gray), "{}", ucl_object_tostring(elt));
 					break;
 				default:
-					colorized_action = fmt::format(fmt::emphasis::bold, ucl_object_tostring(elt));
+					colorized_action = fmt::format(fmt::emphasis::bold, "{}", ucl_object_tostring(elt));
 					break;
 				}
 
 				if (humanreport) {
-					fmt::print(out, ",action={}:{}", act.value(), colorized_action);
+					rspamc_print(out, ",action={}:{}", act.value(), colorized_action);
 				}
 				else {
-					fmt::print(out, "Action: {}\n", colorized_action);
+					rspamc_print(out, "Action: {}\n", colorized_action);
 				}
 			}
 
 			is_spam = act.value() < METRIC_ACTION_GREYLIST ? true : false;
 			if (!humanreport) {
-				fmt::print(out, "Spam: {}\n", is_spam ? "true" : "false");
+				rspamc_print(out, "Spam: {}\n", is_spam ? "true" : "false");
 			}
 		}
 		else {
 			if (humanreport) {
-				fmt::print(out, ",action={}:{}", METRIC_ACTION_NOACTION, ucl_object_tostring(elt));
+				rspamc_print(out, ",action={}:{}", METRIC_ACTION_NOACTION, ucl_object_tostring(elt));
 			}
 			else {
 				print_protocol_string("action", "Action");
@@ -1066,21 +1106,21 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 			is_skipped = 1;
 		}
 
-		fmt::print(out, ",spam={},skipped={}\n", is_spam ? 1 : 0, is_skipped);
+		rspamc_print(out, ",spam={},skipped={}\n", is_spam ? 1 : 0, is_skipped);
 	}
 	else if (got_scores == 2) {
-		fmt::print(out,
-				   "Score: {} / {}\n",
-				   emphasis_argument(score, 2),
-				   emphasis_argument(required_score, 2));
+		rspamc_print(out,
+					 "Score: {} / {}\n",
+					 emphasis_argument(score, 2),
+					 emphasis_argument(required_score, 2));
 	}
 
 	if (humanreport) {
-		fmt::print(out, "Content analysis details:   ({} points, {} required)\n\n",
-				   emphasis_argument(score, 2),
-				   emphasis_argument(addheader_score, 2));
-		fmt::print(out, " pts rule name              description\n");
-		fmt::print(out, "---- ---------------------- --------------------------------------------------\n");
+		rspamc_print(out, "Content analysis details:   ({} points, {} required)\n\n",
+					 emphasis_argument(score, 2),
+					 emphasis_argument(addheader_score, 2));
+		rspamc_print(out, " pts rule name              description\n");
+		rspamc_print(out, "---- ---------------------- --------------------------------------------------\n");
 	}
 
 	elt = ucl_object_lookup(obj, "symbols");
@@ -1101,7 +1141,7 @@ rspamc_metric_output(FILE *out, const ucl_object_t *obj)
 		}
 	}
 	if (humanreport) {
-		fmt::print(out, "\n");
+		rspamc_print(out, "\n");
 	}
 }
 
@@ -1122,8 +1162,8 @@ rspamc_profile_output(FILE *out, const ucl_object_t *obj)
 					 });
 
 	for (const auto *cur_elt: ar) {
-		fmt::print(out, "\t{}: {:3} usec\n",
-				   ucl_object_key(cur_elt), ucl_object_todouble(cur_elt));
+		rspamc_print(out, "\t{}: {:3} usec\n",
+					 ucl_object_key(cur_elt), ucl_object_todouble(cur_elt));
 	}
 }
 
@@ -1135,7 +1175,7 @@ rspamc_symbols_output(FILE *out, ucl_object_t *obj)
 	auto print_protocol_string = [&](const char *ucl_name, const char *output_message) {
 		auto *elt = ucl_object_lookup(obj, ucl_name);
 		if (elt) {
-			fmt::print(out, "{}: {}\n", output_message, ucl_object_tostring(elt));
+			rspamc_print(out, "{}: {}\n", output_message, ucl_object_tostring(elt));
 		}
 	};
 
@@ -1162,7 +1202,7 @@ rspamc_symbols_output(FILE *out, ucl_object_t *obj)
 			}
 		}
 		else {
-			fmt::print(out, "Urls: {}\n", emitted);
+			rspamc_print(out, "Urls: {}\n", emitted);
 		}
 		free(emitted);
 	}
@@ -1184,7 +1224,7 @@ rspamc_symbols_output(FILE *out, ucl_object_t *obj)
 			}
 		}
 		else {
-			fmt::print(out, "Emails: {}\n", emitted);
+			rspamc_print(out, "Emails: {}\n", emitted);
 		}
 		free(emitted);
 	}
@@ -1201,14 +1241,14 @@ rspamc_symbols_output(FILE *out, ucl_object_t *obj)
 
 		while ((cmesg = ucl_object_iterate(elt, &mit, true)) != nullptr) {
 			if (ucl_object_type(cmesg) == UCL_STRING) {
-				fmt::print(out, "Message - {}: {}\n",
-						   ucl_object_key(cmesg), ucl_object_tostring(cmesg));
+				rspamc_print(out, "Message - {}: {}\n",
+							 ucl_object_key(cmesg), ucl_object_tostring(cmesg));
 			}
 			else {
 				char *rendered_message;
 				rendered_message = (char *) ucl_object_emit(cmesg, UCL_EMIT_JSON_COMPACT);
-				fmt::print(out, "Message - {}: {:.60}\n",
-						   ucl_object_key(cmesg), rendered_message);
+				rspamc_print(out, "Message - {}: {:.60}\n",
+							 ucl_object_key(cmesg), rendered_message);
 				free(rendered_message);
 			}
 		}
@@ -1216,21 +1256,21 @@ rspamc_symbols_output(FILE *out, ucl_object_t *obj)
 
 	elt = ucl_object_lookup(obj, "dkim-signature");
 	if (elt && elt->type == UCL_STRING) {
-		fmt::print(out, "DKIM-Signature: {}\n", ucl_object_tostring(elt));
+		rspamc_print(out, "DKIM-Signature: {}\n", ucl_object_tostring(elt));
 	}
 	else if (elt && elt->type == UCL_ARRAY) {
 		ucl_object_iter_t it = nullptr;
 		const ucl_object_t *cur;
 
 		while ((cur = ucl_object_iterate(elt, &it, true)) != nullptr) {
-			fmt::print(out, "DKIM-Signature: {}\n", ucl_object_tostring(cur));
+			rspamc_print(out, "DKIM-Signature: {}\n", ucl_object_tostring(cur));
 		}
 	}
 
 	elt = ucl_object_lookup(obj, "profile");
 
 	if (elt) {
-		fmt::print(out, "Profile data:\n");
+		rspamc_print(out, "Profile data:\n");
 		rspamc_profile_output(out, elt);
 	}
 }
@@ -1242,34 +1282,34 @@ rspamc_uptime_output(FILE *out, ucl_object_t *obj)
 
 	const auto *elt = ucl_object_lookup(obj, "version");
 	if (elt != nullptr) {
-		fmt::print(out, "Rspamd version: {}\n", ucl_object_tostring(elt));
+		rspamc_print(out, "Rspamd version: {}\n", ucl_object_tostring(elt));
 	}
 
 	elt = ucl_object_lookup(obj, "uptime");
 	if (elt != nullptr) {
-		fmt::print("Uptime: ");
+		rspamc_print(out, "Uptime: ");
 		seconds = ucl_object_toint(elt);
 		if (seconds >= 2 * 3600) {
 			days = seconds / 86400;
 			hours = seconds / 3600 - days * 24;
 			minutes = seconds / 60 - hours * 60 - days * 1440;
-			fmt::print("{} day{} {} hour{} {} minute{}\n", days,
-					   days > 1 ? "s" : "", hours, hours > 1 ? "s" : "",
-					   minutes, minutes > 1 ? "s" : "");
+			rspamc_print(out, "{} day{} {} hour{} {} minute{}\n", days,
+						 days > 1 ? "s" : "", hours, hours > 1 ? "s" : "",
+						 minutes, minutes > 1 ? "s" : "");
 		}
 		/* If uptime is less than 1 minute print only seconds */
 		else if (seconds / 60 == 0) {
-			fmt::print("{} second{}\n", seconds,
-					   seconds > 1 ? "s" : "");
+			rspamc_print(out, "{} second{}\n", seconds,
+						 seconds > 1 ? "s" : "");
 		}
 		/* Else print the minutes and seconds. */
 		else {
 			hours = seconds / 3600;
 			minutes = seconds / 60 - hours * 60;
 			seconds -= hours * 3600 + minutes * 60;
-			fmt::print("{} hour {} minute{} {} second{}\n", hours,
-					   minutes, minutes > 1 ? "s" : "",
-					   seconds, seconds > 1 ? "s" : "");
+			rspamc_print(out, "{} hour {} minute{} {} second{}\n", hours,
+						 minutes, minutes > 1 ? "s" : "",
+						 seconds, seconds > 1 ? "s" : "");
 		}
 	}
 }
@@ -1278,7 +1318,7 @@ static void
 rspamc_counters_output(FILE *out, ucl_object_t *obj)
 {
 	if (obj->type != UCL_ARRAY) {
-		fmt::print(out, "Bad output\n");
+		rspamc_print(out, "Bad output\n");
 		return;
 	}
 
@@ -1309,24 +1349,24 @@ rspamc_counters_output(FILE *out, ucl_object_t *obj)
 	memset(dash_buf, '-', dashes + max_len);
 	dash_buf[dashes + max_len] = '\0';
 
-	fmt::print(out, "Symbols cache\n");
+	rspamc_print(out, "Symbols cache\n");
 
-	fmt::print(out, " {} \n", emphasis_argument(dash_buf));
-	fmt::print(out,
-			   "| {:<4} | {:<{}} | {:^7} | {:^13} | {:^7} |\n",
-			   "Pri",
-			   "Symbol",
-			   max_len,
-			   "Weight",
-			   "Frequency",
-			   "Hits");
-	fmt::print(out, " {} \n", emphasis_argument(dash_buf));
-	fmt::print(out, "| {:<4} | {:<{}} | {:^7} | {:^13} | {:^7} |\n", "",
-			   "", max_len,
-			   "", "hits/min", "");
+	rspamc_print(out, " {} \n", emphasis_argument(dash_buf));
+	rspamc_print(out,
+				 "| {:<4} | {:<{}} | {:^7} | {:^13} | {:^7} |\n",
+				 "Pri",
+				 "Symbol",
+				 max_len,
+				 "Weight",
+				 "Frequency",
+				 "Hits");
+	rspamc_print(out, " {} \n", emphasis_argument(dash_buf));
+	rspamc_print(out, "| {:<4} | {:<{}} | {:^7} | {:^13} | {:^7} |\n", "",
+				 "", max_len,
+				 "", "hits/min", "");
 
 	for (const auto [i, cur]: rspamd::enumerate(counters_vec)) {
-		fmt::print(out, " {} \n", dash_buf);
+		rspamc_print(out, " {} \n", dash_buf);
 		const auto *sym = ucl_object_lookup(cur, "symbol");
 		const auto *weight = ucl_object_lookup(cur, "weight");
 		const auto *freq = ucl_object_lookup(cur, "frequency");
@@ -1345,16 +1385,16 @@ rspamc_counters_output(FILE *out, ucl_object_t *obj)
 				sym_name = ucl_object_tostring(sym);
 			}
 
-			fmt::print(out, "| {:<4} | {:<{}} | {:^7.1f} | {:^6.3f}({:^5.3f}) | {:^7} |\n", i,
-					   sym_name,
-					   max_len,
-					   ucl_object_todouble(weight),
-					   ucl_object_todouble(freq) * 60.0,
-					   ucl_object_todouble(freq_dev) * 60.0,
-					   (std::uintmax_t) ucl_object_toint(nhits));
+			rspamc_print(out, "| {:<4} | {:<{}} | {:^7.1f} | {:^6.3f}({:^5.3f}) | {:^7} |\n", i,
+						 sym_name,
+						 max_len,
+						 ucl_object_todouble(weight),
+						 ucl_object_todouble(freq) * 60.0,
+						 ucl_object_todouble(freq_dev) * 60.0,
+						 (std::uintmax_t) ucl_object_toint(nhits));
 		}
 	}
-	fmt::print(out, " {} \n", dash_buf);
+	rspamc_print(out, " {} \n", dash_buf);
 }
 
 static void
@@ -1531,7 +1571,7 @@ rspamc_stat_output(FILE *out, ucl_object_t *obj)
 	fmt::format_to(std::back_inserter(out_str), "Total learns: {}\n",
 				   ucl_object_toint(ucl_object_lookup(obj, "total_learns")));
 
-	fmt::print(out, "{}", out_str.c_str());
+	rspamc_print(out, "{}", out_str.c_str());
 }
 
 static void
@@ -1540,26 +1580,26 @@ rspamc_output_headers(FILE *out, struct rspamd_http_message *msg)
 	struct rspamd_http_header *h;
 
 	kh_foreach_value(msg->headers, h, {
-		fmt::print(out, "{}: {}\n", std::string_view{h->name.begin, h->name.len},
-				   std::string_view{h->value.begin, h->value.len});
+		rspamc_print(out, "{}: {}\n", std::string_view{h->name.begin, h->name.len},
+					 std::string_view{h->value.begin, h->value.len});
 	});
 
-	fmt::print(out, "\n");
+	rspamc_print(out, "\n");
 }
 
 static void
 rspamc_mime_output(FILE *out, ucl_object_t *result, GString *input,
-				   gdouble time, GError *err)
+				   double time, GError *err)
 {
-	const gchar *action = "no action", *line_end = "\r\n", *p;
-	gdouble score = 0.0, required_score = 0.0;
+	const char *action = "no action", *line_end = "\r\n", *p;
+	double score = 0.0, required_score = 0.0;
 	gboolean is_spam = FALSE;
 	auto nl_type = RSPAMD_TASK_NEWLINES_CRLF;
 
 	auto headers_pos = rspamd_string_find_eoh(input, nullptr);
 
 	if (headers_pos == -1) {
-		fmt::print(stderr, "cannot find end of headers position");
+		rspamc_print(stderr, "cannot find end of headers position");
 		return;
 	}
 
@@ -1704,24 +1744,24 @@ rspamc_mime_output(FILE *out, ucl_object_t *result, GString *input,
 
 	/* Write message */
 	/* Original headers */
-	fmt::print(out, "{}", std::string_view{input->str, (std::size_t) headers_pos});
+	rspamc_print(out, "{}", std::string_view{input->str, (std::size_t) headers_pos});
 	/* Added headers */
-	fmt::print(out, "{}", added_headers);
+	rspamc_print(out, "{}", added_headers);
 	/* Message body */
-	fmt::print(out, "{}", input->str + headers_pos);
+	rspamc_print(out, "{}", input->str + headers_pos);
 }
 
 static void
 rspamc_client_execute_cmd(const struct rspamc_command &cmd, ucl_object_t *result,
-						  GString *input, gdouble time, GError *err)
+						  GString *input, double time, GError *err)
 {
-	gchar **eargv;
-	gint eargc, infd, outfd, errfd;
+	char **eargv;
+	int eargc, infd, outfd, errfd;
 	GError *exec_err = nullptr;
 	GPid cld;
 
 	if (!g_shell_parse_argv(execute, &eargc, &eargv, &err)) {
-		fmt::print(stderr, "Cannot execute {}: {}", execute, err->message);
+		rspamc_print(stderr, "Cannot execute {}: {}", execute, err->message);
 		g_error_free(err);
 
 		return;
@@ -1731,7 +1771,7 @@ rspamc_client_execute_cmd(const struct rspamc_command &cmd, ucl_object_t *result
 								  static_cast<GSpawnFlags>(G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD), nullptr, nullptr, &cld,
 								  &infd, &outfd, &errfd, &exec_err)) {
 
-		fmt::print(stderr, "Cannot execute {}: {}", execute, exec_err->message);
+		rspamc_print(stderr, "Cannot execute {}: {}", execute, exec_err->message);
 		g_error_free(exec_err);
 
 		exit(EXIT_FAILURE);
@@ -1755,7 +1795,7 @@ rspamc_client_execute_cmd(const struct rspamc_command &cmd, ucl_object_t *result
 					ucl_out = (char *) ucl_object_emit(result,
 													   compact ? UCL_EMIT_JSON_COMPACT : UCL_EMIT_CONFIG);
 				}
-				fmt::print(out, "{}", ucl_out);
+				rspamc_print(out, "{}", ucl_out);
 				free(ucl_out);
 			}
 			else {
@@ -1765,7 +1805,7 @@ rspamc_client_execute_cmd(const struct rspamc_command &cmd, ucl_object_t *result
 			ucl_object_unref(result);
 		}
 		else {
-			fmt::print(out, "{}\n", err->message);
+			rspamc_print(out, "{}\n", err->message);
 		}
 
 		fflush(out);
@@ -1779,13 +1819,13 @@ static void
 rspamc_client_cb(struct rspamd_client_connection *conn,
 				 struct rspamd_http_message *msg,
 				 const char *name, ucl_object_t *result, GString *input,
-				 gpointer ud, gdouble start_time, gdouble send_time,
+				 gpointer ud, double start_time, double send_time,
 				 const char *body, gsize bodylen,
 				 GError *err)
 {
 	struct rspamc_callback_data *cbdata = (struct rspamc_callback_data *) ud;
 	FILE *out = stdout;
-	gdouble finish = rspamd_get_ticks(FALSE), diff;
+	double finish = rspamd_get_ticks(FALSE), diff;
 
 	auto &cmd = cbdata->cmd;
 
@@ -1817,14 +1857,14 @@ rspamc_client_cb(struct rspamd_client_connection *conn,
 		else {
 			if (cmd.need_input && !json) {
 				if (!compact && !humanreport) {
-					fmt::print(out, "Results for file: {} ({:.3} seconds)\n",
-							   emphasis_argument(cbdata->filename), diff);
+					rspamc_print(out, "Results for file: {} ({:.3} seconds)\n",
+								 emphasis_argument(cbdata->filename), diff);
 				}
 			}
 			else {
 				if (!compact && !json && !humanreport) {
-					fmt::print(out, "Results for command: {} ({:.3} seconds)\n",
-							   emphasis_argument(cmd.name), diff);
+					rspamc_print(out, "Results for command: {} ({:.3} seconds)\n",
+								 emphasis_argument(cmd.name), diff);
 				}
 			}
 
@@ -1856,7 +1896,7 @@ rspamc_client_cb(struct rspamd_client_connection *conn,
 														   compact ? UCL_EMIT_JSON_COMPACT : UCL_EMIT_CONFIG);
 					}
 
-					fmt::print(out, "{}", ucl_out);
+					rspamc_print(out, "{}", ucl_out);
 					free(ucl_out);
 				}
 				else {
@@ -1864,14 +1904,14 @@ rspamc_client_cb(struct rspamd_client_connection *conn,
 				}
 
 				if (body) {
-					fmt::print(out, "\nNew body:\n{}\n",
-							   std::string_view{body, bodylen});
+					rspamc_print(out, "\nNew body:\n{}\n",
+								 std::string_view{body, bodylen});
 				}
 
 				ucl_object_unref(result);
 			}
 			else if (err != nullptr) {
-				fmt::print(out, "{}\n", err->message);
+				rspamc_print(out, "{}\n", err->message);
 
 				if (json && msg != nullptr) {
 					gsize rawlen;
@@ -1880,11 +1920,11 @@ rspamc_client_cb(struct rspamd_client_connection *conn,
 
 					if (raw_body) {
 						/* We can also output the resulting json */
-						fmt::print(out, "{}\n", std::string_view{raw_body, (std::size_t)(rawlen - bodylen)});
+						rspamc_print(out, "{}\n", std::string_view{raw_body, (std::size_t)(rawlen - bodylen)});
 					}
 				}
 			}
-			fmt::print(out, "\n");
+			rspamc_print(out, "\n");
 		}
 
 		fflush(out);
@@ -1904,7 +1944,7 @@ rspamc_process_input(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 {
 	struct rspamd_client_connection *conn;
 	const char *p;
-	guint16 port;
+	uint16_t port;
 	GError *err = nullptr;
 	std::string hostbuf;
 
@@ -1979,8 +2019,8 @@ rspamc_process_input(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 		}
 	}
 	else {
-		fmt::print(stderr, "cannot connect to {}: {}\n", connect_str,
-				   strerror(errno));
+		rspamc_print(stderr, "cannot connect to {}: {}\n", connect_str,
+					 strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 }
@@ -2069,8 +2109,8 @@ rspamc_process_dir(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 			if (pentry->d_type == DT_UNKNOWN) {
 				/* Fallback to lstat */
 				if (lstat(fpath.c_str(), &st) == -1) {
-					fmt::print(stderr, "cannot stat file {}: {}\n",
-							   fpath, strerror(errno));
+					rspamc_print(stderr, "cannot stat file {}: {}\n",
+								 fpath, strerror(errno));
 					continue;
 				}
 
@@ -2087,8 +2127,8 @@ rspamc_process_dir(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 			}
 #else
 			if (lstat(fpath.c_str(), &st) == -1) {
-				fmt::print(stderr, "cannot stat file {}: {}\n",
-						   fpath, strerror(errno));
+				rspamc_print(stderr, "cannot stat file {}: {}\n",
+							 fpath, strerror(errno));
 				continue;
 			}
 
@@ -2102,8 +2142,8 @@ rspamc_process_dir(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 			else if (is_reg) {
 				auto *in = fopen(fpath.c_str(), "r");
 				if (in == nullptr) {
-					fmt::print(stderr, "cannot open file {}: {}\n",
-							   fpath, strerror(errno));
+					rspamc_print(stderr, "cannot open file {}: {}\n",
+								 fpath, strerror(errno));
 					continue;
 				}
 
@@ -2120,7 +2160,7 @@ rspamc_process_dir(struct ev_loop *ev_base, const struct rspamc_command &cmd,
 		}
 	}
 	else {
-		fmt::print(stderr, "cannot open directory {}: {}\n", name, strerror(errno));
+		rspamc_print(stderr, "cannot open directory {}: {}\n", name, strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -2167,8 +2207,8 @@ int main(int argc, char **argv, char **env)
 			exclude_compiled[i] = g_pattern_spec_new(exclude_patterns[i]);
 
 			if (exclude_compiled[i] == nullptr) {
-				fmt::print(stderr, "Invalid glob pattern: {}\n",
-						   exclude_patterns[i]);
+				rspamc_print(stderr, "Invalid glob pattern: {}\n",
+							 exclude_patterns[i]);
 				exit(EXIT_FAILURE);
 			}
 		}
@@ -2223,7 +2263,7 @@ int main(int argc, char **argv, char **env)
 			/* In case of command read arguments starting from 2 */
 			if (cmd.cmd == RSPAMC_COMMAND_ADD_SYMBOL || cmd.cmd == RSPAMC_COMMAND_ADD_ACTION) {
 				if (argc < 4 || argc > 5) {
-					fmt::print(stderr, "invalid arguments\n");
+					rspamc_print(stderr, "invalid arguments\n");
 					exit(EXIT_FAILURE);
 				}
 				if (argc == 5) {
@@ -2248,14 +2288,14 @@ int main(int argc, char **argv, char **env)
 	}
 
 	if (!maybe_cmd.has_value()) {
-		fmt::print(stderr, "invalid command\n");
+		rspamc_print(stderr, "invalid command\n");
 		exit(EXIT_FAILURE);
 	}
 
 	add_options(kwattrs);
 	auto cmd = maybe_cmd.value();
 
-	if (start_argc == argc) {
+	if (start_argc == argc && files_list == nullptr) {
 		/* Do command without input or with stdin */
 		if (empty_input) {
 			rspamc_process_input(event_loop, cmd, nullptr, "empty", kwattrs);
@@ -2267,29 +2307,57 @@ int main(int argc, char **argv, char **env)
 	else {
 		auto cur_req = 0;
 
+		/* Process files from arguments and `files_list` */
+		std::vector<std::string> files;
+		files.reserve(argc - start_argc);
+
 		for (auto i = start_argc; i < argc; i++) {
+			files.emplace_back(argv[i]);
+		}
+
+		/* If we have list of files, read it and enrich our list */
+		if (files_list) {
+			std::ifstream in_files(files_list);
+			if (!in_files.is_open()) {
+				rspamc_print(stderr, "cannot open file {}\n", files_list);
+				exit(EXIT_FAILURE);
+			}
+			std::string line;
+			while (std::getline(in_files, line)) {
+				/* Trim spaces before inserting */
+				line.erase(0, line.find_first_not_of(" \n\r\t"));
+				line.erase(line.find_last_not_of(" \n\r\t") + 1);
+
+				/* Ignore empty lines */
+				if (!line.empty()) {
+					files.emplace_back(line);
+				}
+			}
+		}
+
+		for (const auto &file: files) {
 			if (cmd.cmd == RSPAMC_COMMAND_FUZZY_DELHASH) {
-				add_client_header(kwattrs, "Hash", argv[i]);
+				add_client_header(kwattrs, "Hash", file.c_str());
 			}
 			else {
 				struct stat st;
 
-				if (stat(argv[i], &st) == -1) {
-					fmt::print(stderr, "cannot stat file {}\n", argv[i]);
+				if (stat(file.c_str(), &st) == -1) {
+					rspamc_print(stderr, "cannot stat file {}\n", file);
 					exit(EXIT_FAILURE);
 				}
 				if (S_ISDIR(st.st_mode)) {
 					/* Directories are processed with a separate limit */
-					rspamc_process_dir(event_loop, cmd, argv[i], kwattrs);
+					rspamc_process_dir(event_loop, cmd, file.c_str(), kwattrs);
 					cur_req = 0;
 				}
 				else {
-					in = fopen(argv[i], "r");
+					in = fopen(file.c_str(), "r");
 					if (in == nullptr) {
-						fmt::print(stderr, "cannot open file {}\n", argv[i]);
+						rspamc_print(stderr, "cannot open file {}\n", file);
 						exit(EXIT_FAILURE);
 					}
-					rspamc_process_input(event_loop, cmd, in, argv[i], kwattrs);
+					rspamc_process_input(event_loop, cmd, in, file.c_str(), kwattrs);
 					cur_req++;
 					fclose(in);
 				}
@@ -2316,8 +2384,8 @@ int main(int argc, char **argv, char **env)
 	for (auto cld: children) {
 		auto res = 0;
 		if (waitpid(cld, &res, 0) == -1) {
-			fmt::print(stderr, "Cannot wait for {}: {}", cld,
-					   strerror(errno));
+			rspamc_print(stderr, "Cannot wait for {}: {}", cld,
+						 strerror(errno));
 
 			ret = errno;
 		}
