@@ -20,15 +20,15 @@
 #include "ref.h"
 #include "cryptobox.h"
 
-#ifdef  __cplusplus
+#ifdef __cplusplus
 extern "C" {
 #endif
 /*
  * KEX cached data
  */
 struct rspamd_cryptobox_nm {
-	guchar nm[rspamd_cryptobox_MAX_NMBYTES];
-	guint64 sk_id; /* Used to store secret key id */
+	unsigned char nm[rspamd_cryptobox_MAX_NMBYTES];
+	uint64_t sk_id; /* Used to store secret key id */
 	ref_entry_t ref;
 };
 
@@ -36,7 +36,7 @@ struct rspamd_cryptobox_nm {
  * Generic keypair
  */
 struct rspamd_cryptobox_keypair {
-	guchar id[rspamd_cryptobox_HASHBYTES];
+	unsigned char id[rspamd_cryptobox_HASHBYTES];
 	enum rspamd_cryptobox_keypair_type type;
 	enum rspamd_cryptobox_mode alg;
 	ucl_object_t *extensions;
@@ -46,48 +46,48 @@ struct rspamd_cryptobox_keypair {
 /*
  * NIST p256 ecdh keypair
  */
-#define RSPAMD_CRYPTOBOX_KEYPAIR_NIST(x) ((struct rspamd_cryptobox_keypair_nist *)(x))
+#define RSPAMD_CRYPTOBOX_KEYPAIR_NIST(x) ((struct rspamd_cryptobox_keypair_nist *) (x))
 struct rspamd_cryptobox_keypair_nist {
 	struct rspamd_cryptobox_keypair parent;
-	guchar sk[32];
-	guchar pk[65];
+	unsigned char sk[32];
+	unsigned char pk[65];
 };
 
 /*
  * Curve25519 ecdh keypair
  */
-#define RSPAMD_CRYPTOBOX_KEYPAIR_25519(x) ((struct rspamd_cryptobox_keypair_25519 *)(x))
+#define RSPAMD_CRYPTOBOX_KEYPAIR_25519(x) ((struct rspamd_cryptobox_keypair_25519 *) (x))
 struct rspamd_cryptobox_keypair_25519 {
 	struct rspamd_cryptobox_keypair parent;
-	guchar sk[32];
-	guchar pk[32];
+	unsigned char sk[32];
+	unsigned char pk[32];
 };
 
 /*
  * NIST p256 ecdsa keypair
  */
-#define RSPAMD_CRYPTOBOX_KEYPAIR_SIG_NIST(x) ((struct rspamd_cryptobox_keypair_sig_nist *)(x))
+#define RSPAMD_CRYPTOBOX_KEYPAIR_SIG_NIST(x) ((struct rspamd_cryptobox_keypair_sig_nist *) (x))
 struct rspamd_cryptobox_keypair_sig_nist {
 	struct rspamd_cryptobox_keypair parent;
-	guchar sk[32];
-	guchar pk[65];
+	unsigned char sk[32];
+	unsigned char pk[65];
 };
 
 /*
  * Ed25519 keypair
  */
-#define RSPAMD_CRYPTOBOX_KEYPAIR_SIG_25519(x) ((struct rspamd_cryptobox_keypair_sig_25519 *)(x))
+#define RSPAMD_CRYPTOBOX_KEYPAIR_SIG_25519(x) ((struct rspamd_cryptobox_keypair_sig_25519 *) (x))
 struct rspamd_cryptobox_keypair_sig_25519 {
 	struct rspamd_cryptobox_keypair parent;
-	guchar sk[64];
-	guchar pk[32];
+	unsigned char sk[64];
+	unsigned char pk[32];
 };
 
 /*
  * Public component of the keypair
  */
 struct rspamd_cryptobox_pubkey {
-	guchar id[rspamd_cryptobox_HASHBYTES];
+	unsigned char id[rspamd_cryptobox_HASHBYTES];
 	struct rspamd_cryptobox_nm *nm;
 	enum rspamd_cryptobox_keypair_type type;
 	enum rspamd_cryptobox_mode alg;
@@ -97,46 +97,46 @@ struct rspamd_cryptobox_pubkey {
 /*
  * Public p256 ecdh
  */
-#define RSPAMD_CRYPTOBOX_PUBKEY_NIST(x) ((struct rspamd_cryptobox_pubkey_nist *)(x))
+#define RSPAMD_CRYPTOBOX_PUBKEY_NIST(x) ((struct rspamd_cryptobox_pubkey_nist *) (x))
 struct rspamd_cryptobox_pubkey_nist {
 	struct rspamd_cryptobox_pubkey parent;
-	guchar pk[65];
+	unsigned char pk[65];
 };
 
 /*
  * Public curve25519 ecdh
  */
-#define RSPAMD_CRYPTOBOX_PUBKEY_25519(x) ((struct rspamd_cryptobox_pubkey_25519 *)(x))
+#define RSPAMD_CRYPTOBOX_PUBKEY_25519(x) ((struct rspamd_cryptobox_pubkey_25519 *) (x))
 struct rspamd_cryptobox_pubkey_25519 {
 	struct rspamd_cryptobox_pubkey parent;
-	guchar pk[32];
+	unsigned char pk[32];
 };
 
 /*
  * Public p256 ecdsa
  */
-#define RSPAMD_CRYPTOBOX_PUBKEY_SIG_NIST(x) ((struct rspamd_cryptobox_pubkey_sig_nist *)(x))
+#define RSPAMD_CRYPTOBOX_PUBKEY_SIG_NIST(x) ((struct rspamd_cryptobox_pubkey_sig_nist *) (x))
 struct rspamd_cryptobox_pubkey_sig_nist {
 	struct rspamd_cryptobox_pubkey parent;
-	guchar pk[65];
+	unsigned char pk[65];
 };
 
 /*
  * Public ed25519
  */
-#define RSPAMD_CRYPTOBOX_PUBKEY_SIG_25519(x) ((struct rspamd_cryptobox_pubkey_sig_25519 *)(x))
+#define RSPAMD_CRYPTOBOX_PUBKEY_SIG_25519(x) ((struct rspamd_cryptobox_pubkey_sig_25519 *) (x))
 struct rspamd_cryptobox_pubkey_sig_25519 {
 	struct rspamd_cryptobox_pubkey parent;
-	guchar pk[32];
+	unsigned char pk[32];
 };
 
-void rspamd_cryptobox_nm_dtor (struct rspamd_cryptobox_nm *nm);
+void rspamd_cryptobox_nm_dtor(struct rspamd_cryptobox_nm *nm);
 
-void rspamd_cryptobox_keypair_dtor (struct rspamd_cryptobox_keypair *kp);
+void rspamd_cryptobox_keypair_dtor(struct rspamd_cryptobox_keypair *kp);
 
-void rspamd_cryptobox_pubkey_dtor (struct rspamd_cryptobox_pubkey *p);
+void rspamd_cryptobox_pubkey_dtor(struct rspamd_cryptobox_pubkey *p);
 
-#ifdef  __cplusplus
+#ifdef __cplusplus
 }
 #endif
 
