@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -25,8 +25,10 @@
 #include "unix-std.h"
 #include "protocol_internal.h"
 #include "libserver/mempool_vars_internal.h"
+#include "libserver/worker_util.h"
 #include "contrib/fastutf8/fastutf8.h"
 #include "task.h"
+#include "lua/lua_classnames.h"
 #include <math.h>
 
 #ifdef SYS_ZSTD
@@ -63,11 +65,11 @@ rspamd_protocol_quark(void)
 /*
  * Remove <> from the fixed string and copy it to the pool
  */
-static gchar *
+static char *
 rspamd_protocol_escape_braces(struct rspamd_task *task, rspamd_ftok_t *in)
 {
-	guint nchars = 0;
-	const gchar *p;
+	unsigned int nchars = 0;
+	const char *p;
 	rspamd_ftok_t tok;
 	gboolean has_obrace = FALSE;
 
@@ -111,7 +113,7 @@ rspamd_protocol_handle_url(struct rspamd_task *task,
 	GHashTable *query_args;
 	GHashTableIter it;
 	struct http_parser_url u;
-	const gchar *p;
+	const char *p;
 	gsize pathlen;
 	rspamd_ftok_t *key, *value;
 	gpointer k, v;
@@ -209,6 +211,19 @@ rspamd_protocol_handle_url(struct rspamd_task *task,
 			goto err;
 		}
 		break;
+	case 'M':
+	case 'm':
+		/* metrics, process */
+		if (COMPARE_CMD(p, MSG_CMD_METRICS, pathlen)) {
+			msg_debug_protocol("got metrics command");
+			task->cmd = CMD_METRICS;
+			task->flags |= RSPAMD_TASK_FLAG_SKIP;
+			task->processed_stages |= RSPAMD_TASK_STAGE_DONE; /* Skip all */
+		}
+		else {
+			goto err;
+		}
+		break;
 	default:
 		goto err;
 	}
@@ -221,7 +236,7 @@ rspamd_protocol_handle_url(struct rspamd_task *task,
 		g_hash_table_iter_init(&it, query_args);
 
 		while (g_hash_table_iter_next(&it, &k, &v)) {
-			gchar *key_cpy;
+			char *key_cpy;
 			key = k;
 			value = v;
 
@@ -253,7 +268,7 @@ rspamd_protocol_process_recipients(struct rspamd_task *task,
 		quoted_string,
 		normal_string,
 	} state = skip_spaces;
-	const gchar *p, *end, *start_addr;
+	const char *p, *end, *start_addr;
 	struct rspamd_email_address *addr;
 
 	p = hdr->begin;
@@ -374,7 +389,7 @@ rspamd_protocol_process_recipients(struct rspamd_task *task,
 	} while (0)
 
 static void
-rspamd_protocol_handle_flag(struct rspamd_task *task, const gchar *str,
+rspamd_protocol_handle_flag(struct rspamd_task *task, const char *str,
 							gsize len)
 {
 	gboolean known = FALSE;
@@ -394,7 +409,7 @@ rspamd_protocol_handle_flag(struct rspamd_task *task, const gchar *str,
 	CHECK_PROTOCOL_FLAG("groups", RSPAMD_TASK_PROTOCOL_FLAG_GROUPS);
 
 	if (!known) {
-		msg_warn_protocol("unknown flag: %*s", (gint) len, str);
+		msg_warn_protocol("unknown flag: %*s", (int) len, str);
 	}
 }
 
@@ -408,7 +423,7 @@ rspamd_protocol_process_flags(struct rspamd_task *task, const rspamd_ftok_t *hdr
 		skip_spaces,
 		read_flag,
 	} state = skip_spaces;
-	const gchar *p, *end, *start;
+	const char *p, *end, *start;
 
 	p = hdr->begin;
 	end = hdr->begin + hdr->len;
@@ -459,7 +474,7 @@ rspamd_protocol_handle_headers(struct rspamd_task *task,
 	rspamd_ftok_t *hn_tok, *hv_tok, srch;
 	gboolean has_ip = FALSE, seen_settings_header = FALSE;
 	struct rspamd_http_header *header, *h;
-	gchar *ntok;
+	char *ntok;
 
 	kh_foreach_value (msg->headers, header, {
 		DL_FOREACH (header, h) {
@@ -704,7 +719,7 @@ rspamd_protocol_handle_headers(struct rspamd_task *task,
 		}
 		IF_HEADER(MTA_TAG_HEADER)
 		{
-			gchar *mta_tag;
+			char *mta_tag;
 			mta_tag = rspamd_mempool_ftokdup(task->task_pool, hv_tok);
 			rspamd_mempool_set_variable(task->task_pool,
 										RSPAMD_MEMPOOL_MTA_TAG,
@@ -713,7 +728,7 @@ rspamd_protocol_handle_headers(struct rspamd_task *task,
 		}
 		IF_HEADER(MTA_NAME_HEADER)
 		{
-			gchar *mta_name;
+			char *mta_name;
 			mta_name = rspamd_mempool_ftokdup(task->task_pool, hv_tok);
 			rspamd_mempool_set_variable(task->task_pool,
 										RSPAMD_MEMPOOL_MTA_NAME,
@@ -773,11 +788,11 @@ rspamd_protocol_parse_task_flags(rspamd_mempool_t *pool,
 								 GError **err)
 {
 	struct rspamd_rcl_struct_parser *pd = ud;
-	gint *target;
-	const gchar *key;
+	int *target;
+	const char *key;
 	gboolean value;
 
-	target = (gint *) (((gchar *) pd->user_struct) + pd->offset);
+	target = (int *) (((char *) pd->user_struct) + pd->offset);
 	key = ucl_object_key(obj);
 	value = ucl_object_toboolean(obj);
 
@@ -793,65 +808,66 @@ rspamd_protocol_parse_task_flags(rspamd_mempool_t *pool,
 	return TRUE;
 }
 
-static struct rspamd_rcl_section *control_parser = NULL;
+static struct rspamd_rcl_sections_map *control_parser = NULL;
 
-static void
-rspamd_protocol_control_parser_init(void)
+RSPAMD_CONSTRUCTOR(rspamd_protocol_control_parser_ctor)
 {
-	struct rspamd_rcl_section *sub;
 
-	if (control_parser == NULL) {
-		sub = rspamd_rcl_add_section(&control_parser,
-									 "*",
-									 NULL,
-									 NULL,
-									 UCL_OBJECT,
-									 FALSE,
-									 TRUE);
-		/* Default handlers */
-		rspamd_rcl_add_default_handler(sub,
-									   "ip",
-									   rspamd_rcl_parse_struct_addr,
-									   G_STRUCT_OFFSET(struct rspamd_task, from_addr),
-									   0,
-									   NULL);
-		rspamd_rcl_add_default_handler(sub,
-									   "from",
-									   rspamd_rcl_parse_struct_mime_addr,
-									   G_STRUCT_OFFSET(struct rspamd_task, from_envelope),
-									   0,
-									   NULL);
-		rspamd_rcl_add_default_handler(sub,
-									   "rcpt",
-									   rspamd_rcl_parse_struct_mime_addr,
-									   G_STRUCT_OFFSET(struct rspamd_task, rcpt_envelope),
-									   0,
-									   NULL);
-		rspamd_rcl_add_default_handler(sub,
-									   "helo",
-									   rspamd_rcl_parse_struct_string,
-									   G_STRUCT_OFFSET(struct rspamd_task, helo),
-									   0,
-									   NULL);
-		rspamd_rcl_add_default_handler(sub,
-									   "user",
-									   rspamd_rcl_parse_struct_string,
-									   G_STRUCT_OFFSET(struct rspamd_task, auth_user),
-									   0,
-									   NULL);
-		rspamd_rcl_add_default_handler(sub,
-									   "pass_all",
-									   rspamd_protocol_parse_task_flags,
-									   G_STRUCT_OFFSET(struct rspamd_task, flags),
-									   0,
-									   NULL);
-		rspamd_rcl_add_default_handler(sub,
-									   "json",
-									   rspamd_protocol_parse_task_flags,
-									   G_STRUCT_OFFSET(struct rspamd_task, flags),
-									   0,
-									   NULL);
-	}
+	struct rspamd_rcl_section *sub = rspamd_rcl_add_section(&control_parser, NULL,
+															"*",
+															NULL,
+															NULL,
+															UCL_OBJECT,
+															FALSE,
+															TRUE);
+	/* Default handlers */
+	rspamd_rcl_add_default_handler(sub,
+								   "ip",
+								   rspamd_rcl_parse_struct_addr,
+								   G_STRUCT_OFFSET(struct rspamd_task, from_addr),
+								   0,
+								   NULL);
+	rspamd_rcl_add_default_handler(sub,
+								   "from",
+								   rspamd_rcl_parse_struct_mime_addr,
+								   G_STRUCT_OFFSET(struct rspamd_task, from_envelope),
+								   0,
+								   NULL);
+	rspamd_rcl_add_default_handler(sub,
+								   "rcpt",
+								   rspamd_rcl_parse_struct_mime_addr,
+								   G_STRUCT_OFFSET(struct rspamd_task, rcpt_envelope),
+								   0,
+								   NULL);
+	rspamd_rcl_add_default_handler(sub,
+								   "helo",
+								   rspamd_rcl_parse_struct_string,
+								   G_STRUCT_OFFSET(struct rspamd_task, helo),
+								   0,
+								   NULL);
+	rspamd_rcl_add_default_handler(sub,
+								   "user",
+								   rspamd_rcl_parse_struct_string,
+								   G_STRUCT_OFFSET(struct rspamd_task, auth_user),
+								   0,
+								   NULL);
+	rspamd_rcl_add_default_handler(sub,
+								   "pass_all",
+								   rspamd_protocol_parse_task_flags,
+								   G_STRUCT_OFFSET(struct rspamd_task, flags),
+								   0,
+								   NULL);
+	rspamd_rcl_add_default_handler(sub,
+								   "json",
+								   rspamd_protocol_parse_task_flags,
+								   G_STRUCT_OFFSET(struct rspamd_task, flags),
+								   0,
+								   NULL);
+}
+
+RSPAMD_DESTRUCTOR(rspamd_protocol_control_parser_dtor)
+{
+	rspamd_rcl_sections_free(control_parser);
 }
 
 gboolean
@@ -860,8 +876,6 @@ rspamd_protocol_handle_control(struct rspamd_task *task,
 {
 	GError *err = NULL;
 
-	rspamd_protocol_control_parser_init();
-
 	if (!rspamd_rcl_parse(control_parser, task->cfg, task, task->task_pool,
 						  control, &err)) {
 		msg_warn_protocol("cannot parse control block: %e", err);
@@ -909,7 +923,7 @@ struct tree_cb_data {
 static ucl_object_t *
 rspamd_protocol_extended_url(struct rspamd_task *task,
 							 struct rspamd_url *url,
-							 const gchar *encoded, gsize enclen)
+							 const char *encoded, gsize enclen)
 {
 	ucl_object_t *obj, *elt;
 
@@ -958,9 +972,9 @@ urls_protocol_cb(struct rspamd_url *url, struct tree_cb_data *cb)
 {
 	ucl_object_t *obj;
 	struct rspamd_task *task = cb->task;
-	const gchar *user_field = "unknown", *encoded = NULL;
+	const char *user_field = "unknown", *encoded = NULL;
 	gboolean has_user = FALSE;
-	guint len = 0;
+	unsigned int len = 0;
 	gsize enclen = 0;
 
 	if (!(task->protocol_flags & RSPAMD_TASK_PROTOCOL_FLAG_EXT_URLS)) {
@@ -1014,7 +1028,7 @@ urls_protocol_cb(struct rspamd_url *url, struct tree_cb_data *cb)
 								  has_user ? "user" : "from",
 								  len, user_field,
 								  rspamd_inet_address_to_string(task->from_addr),
-								  (gint) enclen, encoded);
+								  (int) enclen, encoded);
 	}
 }
 
@@ -1078,12 +1092,12 @@ rspamd_emails_tree_ucl(khash_t(rspamd_url_hash) * set,
 
 
 /* Write new subject */
-static const gchar *
+static const char *
 rspamd_protocol_rewrite_subject(struct rspamd_task *task)
 {
 	GString *subj_buf;
-	gchar *res;
-	const gchar *s, *c, *p;
+	char *res;
+	const char *s, *c, *p;
 	gsize slen = 0;
 
 	c = rspamd_mempool_get_variable(task->task_pool, "metric_subject");
@@ -1157,7 +1171,7 @@ static ucl_object_t *
 rspamd_metric_symbol_ucl(struct rspamd_task *task, struct rspamd_symbol_result *sym)
 {
 	ucl_object_t *obj = NULL, *ar;
-	const gchar *description = NULL;
+	const char *description = NULL;
 	struct rspamd_symbol_option *opt;
 
 	if (sym->sym != NULL) {
@@ -1200,7 +1214,7 @@ rspamd_metric_symbol_ucl(struct rspamd_task *task, struct rspamd_symbol_result *
 
 static ucl_object_t *
 rspamd_metric_group_ucl(struct rspamd_task *task,
-						struct rspamd_symbols_group *gr, gdouble score)
+						struct rspamd_symbols_group *gr, double score)
 {
 	ucl_object_t *obj = NULL;
 
@@ -1224,7 +1238,7 @@ rspamd_scan_result_ucl(struct rspamd_task *task,
 	gboolean is_spam;
 	struct rspamd_action *action;
 	ucl_object_t *obj = NULL, *sobj;
-	const gchar *subject;
+	const char *subject;
 	struct rspamd_passthrough_result *pr = NULL;
 
 	action = rspamd_check_action_metric(task, &pr, NULL);
@@ -1322,15 +1336,13 @@ rspamd_scan_result_ucl(struct rspamd_task *task,
 			sobj = rspamd_metric_symbol_ucl(task, sym);
 			ucl_object_insert_key(obj, sobj, sym->name, 0, false);
 		}
-	})
+	});
 
-		if (task->cmd != CMD_CHECK)
-	{
+	if (task->cmd != CMD_CHECK) {
 		/* For checkv2 we insert symbols as a separate object */
 		ucl_object_insert_key(top, obj, "symbols", 0, false);
 	}
-	else
-	{
+	else {
 		/* For legacy check we just insert it as "default" all together */
 		ucl_object_insert_key(top, obj, DEFAULT_METRIC, 0, false);
 	}
@@ -1338,7 +1350,7 @@ rspamd_scan_result_ucl(struct rspamd_task *task,
 	/* Handle groups if needed */
 	if (task->protocol_flags & RSPAMD_TASK_PROTOCOL_FLAG_GROUPS) {
 		struct rspamd_symbols_group *gr;
-		gdouble gr_score;
+		double gr_score;
 
 		obj = ucl_object_typed_new(UCL_OBJECT);
 		ucl_object_reserve(obj, kh_size(mres->sym_groups));
@@ -1462,7 +1474,7 @@ rspamd_protocol_output_profiling(struct rspamd_task *task,
 	GHashTableIter it;
 	gpointer k, v;
 	ucl_object_t *prof;
-	gdouble val;
+	double val;
 
 	prof = ucl_object_typed_new(UCL_OBJECT);
 	tbl = rspamd_mempool_get_variable(task->task_pool, "profile");
@@ -1471,7 +1483,7 @@ rspamd_protocol_output_profiling(struct rspamd_task *task,
 		g_hash_table_iter_init(&it, tbl);
 
 		while (g_hash_table_iter_next(&it, &k, &v)) {
-			val = *(gdouble *) v;
+			val = *(double *) v;
 			ucl_object_insert_key(prof, ucl_object_fromdouble(val),
 								  (const char *) k, 0, false);
 		}
@@ -1636,14 +1648,14 @@ rspamd_protocol_write_ucl(struct rspamd_task *task,
 }
 
 void rspamd_protocol_http_reply(struct rspamd_http_message *msg,
-								struct rspamd_task *task, ucl_object_t **pobj)
+								struct rspamd_task *task, ucl_object_t **pobj, int how)
 {
 	struct rspamd_scan_result *metric_res;
 	const struct rspamd_re_cache_stat *restat;
 
 	ucl_object_t *top = NULL;
 	rspamd_fstring_t *reply;
-	gint flags = RSPAMD_PROTOCOL_DEFAULT;
+	int flags = RSPAMD_PROTOCOL_DEFAULT;
 	struct rspamd_action *action;
 
 	/* Removed in 2.0 */
@@ -1668,7 +1680,9 @@ void rspamd_protocol_http_reply(struct rspamd_http_message *msg,
 	}
 
 	if (!(task->flags & RSPAMD_TASK_FLAG_NO_LOG)) {
-		rspamd_roll_history_update(task->worker->srv->history, task);
+		if (task->worker->srv->history) {
+			rspamd_roll_history_update(task->worker->srv->history, task);
+		}
 	}
 	else {
 		msg_debug_protocol("skip history update due to no log flag");
@@ -1695,7 +1709,7 @@ void rspamd_protocol_http_reply(struct rspamd_http_message *msg,
 
 	if (msg->method < HTTP_SYMBOLS && !RSPAMD_TASK_IS_SPAMC(task)) {
 		msg_debug_protocol("writing json reply");
-		rspamd_ucl_emit_fstring(top, UCL_EMIT_JSON_COMPACT, &reply);
+		rspamd_ucl_emit_fstring(top, how, &reply);
 	}
 	else {
 		if (RSPAMD_TASK_IS_SPAMC(task)) {
@@ -1722,7 +1736,7 @@ void rspamd_protocol_http_reply(struct rspamd_http_message *msg,
 
 			/* In case of milter, we append just body, otherwise - full message */
 			if (task->protocol_flags & RSPAMD_TASK_PROTOCOL_FLAG_MILTER) {
-				const gchar *start;
+				const char *start;
 				goffset len, hdr_off;
 
 				start = task->msg.begin;
@@ -1816,7 +1830,7 @@ void rspamd_protocol_http_reply(struct rspamd_http_message *msg,
 
 		if (task->cfg->libs_ctx->out_dict &&
 			task->cfg->libs_ctx->out_dict->id != 0) {
-			gchar dict_str[32];
+			char dict_str[32];
 
 			rspamd_snprintf(dict_str, sizeof(dict_str), "%ud",
 							task->cfg->libs_ctx->out_dict->id);
@@ -1867,7 +1881,7 @@ end:
 #endif
 
 		/* Set average processing time */
-		guint32 slot;
+		uint32_t slot;
 		float processing_time = task->time_real_finish - task->task_timestamp;
 
 #ifndef HAVE_ATOMIC_BUILTINS
@@ -1889,8 +1903,8 @@ void rspamd_protocol_write_log_pipe(struct rspamd_task *task)
 	lua_State *L = task->cfg->lua_state;
 	struct rspamd_scan_result *mres;
 	struct rspamd_symbol_result *sym;
-	gint id, i;
-	guint32 n = 0, nextra = 0;
+	int id, i;
+	uint32_t n = 0, nextra = 0;
 	gsize sz;
 	GArray *extra;
 	struct rspamd_protocol_log_symbol_result er;
@@ -1924,7 +1938,7 @@ void rspamd_protocol_write_log_pipe(struct rspamd_task *task)
 				if (lua_isfunction(L, -1)) {
 					ptask = lua_newuserdata(L, sizeof(*ptask));
 					*ptask = task;
-					rspamd_lua_setclass(L, "rspamd{task}", -1);
+					rspamd_lua_setclass(L, rspamd_task_classname, -1);
 					/* stack:
 					 * -1: task
 					 * -2: func
@@ -2098,11 +2112,12 @@ void rspamd_protocol_write_log_pipe(struct rspamd_task *task)
 	g_array_free(extra, TRUE);
 }
 
-void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
+void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout, struct rspamd_main *srv)
 {
 	struct rspamd_http_message *msg;
-	const gchar *ctype = "application/json";
+	const char *ctype = "application/json";
 	rspamd_fstring_t *reply;
+	ev_tstamp now = ev_time();
 
 	msg = rspamd_http_new_message(HTTP_RESPONSE);
 
@@ -2111,6 +2126,16 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
 						  MESSAGE_FIELD_CHECK(task, message_id));
 	}
 
+	const rspamd_ftok_t *accept_hdr;
+	int out_type = UCL_EMIT_JSON_COMPACT;
+	accept_hdr = rspamd_task_get_request_header(task, "Accept");
+
+	if (accept_hdr && rspamd_substring_search(accept_hdr->begin, accept_hdr->len,
+											  "application/msgpack", sizeof("application/msgpack") - 1) != -1) {
+		ctype = "application/msgpack";
+		out_type = UCL_EMIT_MSGPACK;
+	}
+
 	/* Compatibility */
 	if (task->cmd == CMD_CHECK_RSPAMC) {
 		msg->method = HTTP_SYMBOLS;
@@ -2134,15 +2159,15 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
 							  ucl_object_fromstring(g_quark_to_string(task->err->domain)),
 							  "error_domain", 0, false);
 		reply = rspamd_fstring_sized_new(256);
-		rspamd_ucl_emit_fstring(top, UCL_EMIT_JSON_COMPACT, &reply);
+		rspamd_ucl_emit_fstring(top, out_type, &reply);
 		ucl_object_unref(top);
 
 		/* We also need to validate utf8 */
-		if (rspamd_fast_utf8_validate(reply->str, reply->len) != 0) {
+		if (out_type != UCL_EMIT_MSGPACK && rspamd_fast_utf8_validate(reply->str, reply->len) != 0) {
 			gsize valid_len;
-			gchar *validated;
+			char *validated;
 
-			/* We copy reply several times here but it should be a rare case */
+			/* We copy reply several times here, but it should be a rare case */
 			validated = rspamd_str_make_utf_valid(reply->str, reply->len,
 												  &valid_len, task->task_pool);
 			rspamd_http_message_set_body(msg, validated, valid_len);
@@ -2153,6 +2178,8 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
 		}
 	}
 	else {
+		rspamd_fstring_t *output;
+		struct rspamd_stat stat_copy;
 		msg->status = rspamd_fstring_new_init("OK", 2);
 
 		switch (task->cmd) {
@@ -2161,7 +2188,7 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
 		case CMD_CHECK_SPAMC:
 		case CMD_SKIP:
 		case CMD_CHECK_V2:
-			rspamd_protocol_http_reply(msg, task, NULL);
+			rspamd_protocol_http_reply(msg, task, NULL, out_type);
 			rspamd_protocol_write_log_pipe(task);
 			break;
 		case CMD_PING:
@@ -2169,6 +2196,15 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
 			rspamd_http_message_set_body(msg, "pong" CRLF, 6);
 			ctype = "text/plain";
 			break;
+		case CMD_METRICS:
+			msg_debug_protocol("writing metrics to client");
+
+			memcpy(&stat_copy, srv->stat, sizeof(stat_copy));
+			output = rspamd_metrics_to_prometheus_string(
+				rspamd_worker_metrics_object(srv->cfg, &stat_copy, now - srv->start_time));
+			rspamd_http_message_set_body_from_fstring_steal(msg, output);
+			ctype = "application/openmetrics-text; version=1.0.0; charset=utf-8";
+			break;
 		default:
 			msg_err_protocol("BROKEN");
 			break;
@@ -2176,7 +2212,7 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout)
 	}
 
 	ev_now_update(task->event_loop);
-	msg->date = ev_time();
+	msg->date = now;
 
 	rspamd_http_connection_reset(task->http_conn);
 	rspamd_http_connection_write_message(task->http_conn, msg, NULL,
