@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -29,6 +29,7 @@
 #include "libserver/cfg_file_private.h"
 #include "libmime/lang_detection.h"
 #include "libmime/scan_result_private.h"
+#include "lua/lua_classnames.h"
 
 #ifdef WITH_JEMALLOC
 #include <jemalloc/jemalloc.h>
@@ -50,11 +51,6 @@ __KHASH_IMPL(rspamd_req_headers_hash, static inline,
 			 rspamd_ftok_t *, struct rspamd_request_header_chain *, 1,
 			 rspamd_ftok_icase_hash, rspamd_ftok_icase_equal)
 
-/*
- * Do not print more than this amount of elts
- */
-static const int max_log_elts = 7;
-
 static GQuark
 rspamd_task_quark(void)
 {
@@ -74,7 +70,7 @@ rspamd_task_new(struct rspamd_worker *worker,
 {
 	struct rspamd_task *new_task;
 	rspamd_mempool_t *task_pool;
-	guint flags = 0;
+	unsigned int flags = RSPAMD_TASK_FLAG_LEARN_AUTO;
 
 	if (pool == NULL) {
 		task_pool = rspamd_mempool_new(rspamd_mempool_suggest_size(),
@@ -137,7 +133,7 @@ rspamd_task_reply(struct rspamd_task *task)
 	}
 	else {
 		if (!(task->processed_stages & RSPAMD_TASK_STAGE_REPLIED)) {
-			rspamd_protocol_write_reply(task, write_timeout);
+			rspamd_protocol_write_reply(task, write_timeout, task->worker->srv);
 		}
 	}
 }
@@ -177,8 +173,8 @@ rspamd_task_fin(void *arg)
 void rspamd_task_free(struct rspamd_task *task)
 {
 	struct rspamd_email_address *addr;
-	static guint free_iters = 0;
-	guint i;
+	static unsigned int free_iters = 0;
+	unsigned int i;
 
 	if (task) {
 		debug_task("free pointer %p", task);
@@ -257,7 +253,7 @@ void rspamd_task_free(struct rspamd_task *task)
 				/* Perform more expensive cleanup cycle */
 				gsize allocated = 0, active = 0, metadata = 0,
 					  resident = 0, mapped = 0, old_lua_mem = 0;
-				gdouble t1, t2;
+				double t1, t2;
 
 				old_lua_mem = lua_gc(task->cfg->lua_state, LUA_GCCOUNT, 0);
 				t1 = rspamd_get_ticks(FALSE);
@@ -284,7 +280,7 @@ void rspamd_task_free(struct rspamd_task *task)
 								old_lua_mem, lua_gc(task->cfg->lua_state, LUA_GCCOUNT, 0),
 								(t2 - t1) * 1000.0);
 				free_iters = rspamd_time_jitter(0,
-												(gdouble) task->cfg->full_gc_iters / 2);
+												(double) task->cfg->full_gc_iters / 2);
 			}
 
 			REF_RELEASE(task->cfg);
@@ -311,7 +307,7 @@ void rspamd_task_free(struct rspamd_task *task)
 struct rspamd_task_map {
 	gpointer begin;
 	gulong len;
-	gint fd;
+	int fd;
 };
 
 static void
@@ -325,19 +321,19 @@ rspamd_task_unmapper(gpointer ud)
 
 gboolean
 rspamd_task_load_message(struct rspamd_task *task,
-						 struct rspamd_http_message *msg, const gchar *start, gsize len)
+						 struct rspamd_http_message *msg, const char *start, gsize len)
 {
-	guint control_len, r;
+	unsigned int control_len, r;
 	struct ucl_parser *parser;
 	ucl_object_t *control_obj;
-	gchar filepath[PATH_MAX], *fp;
-	gint fd, flen;
+	char filepath[PATH_MAX], *fp;
+	int fd, flen;
 	gulong offset = 0, shmem_size = 0;
 	rspamd_ftok_t *tok;
 	gpointer map;
 	struct stat st;
 	struct rspamd_task_map *m;
-	const gchar *ft;
+	const char *ft;
 
 #ifdef HAVE_SANE_SHMEM
 	ft = "shm";
@@ -429,7 +425,7 @@ rspamd_task_load_message(struct rspamd_task *task,
 			}
 		}
 
-		task->msg.begin = ((guchar *) map) + offset;
+		task->msg.begin = ((unsigned char *) map) + offset;
 		task->msg.len = shmem_size;
 		m = rspamd_mempool_alloc(task->task_pool, sizeof(*m));
 		m->begin = map;
@@ -440,86 +436,89 @@ rspamd_task_load_message(struct rspamd_task *task,
 					  fp, shmem_size, offset, fd);
 
 		rspamd_mempool_add_destructor(task->task_pool, rspamd_task_unmapper, m);
-
-		return TRUE;
 	}
+	else {
+		/* Try file */
+		tok = rspamd_task_get_request_header(task, "file");
 
-	tok = rspamd_task_get_request_header(task, "file");
-
-	if (tok == NULL) {
-		tok = rspamd_task_get_request_header(task, "path");
-	}
-
-	if (tok) {
-		debug_task("want to scan file %T", tok);
-
-		r = rspamd_strlcpy(filepath, tok->begin,
-						   MIN(sizeof(filepath), tok->len + 1));
-
-		rspamd_url_decode(filepath, filepath, r + 1);
-		flen = strlen(filepath);
-
-		if (filepath[0] == '"' && flen > 2) {
-			/* We need to unquote filepath */
-			fp = &filepath[1];
-			fp[flen - 2] = '\0';
-		}
-		else {
-			fp = &filepath[0];
+		if (tok == NULL) {
+			tok = rspamd_task_get_request_header(task, "path");
 		}
 
-		if (stat(fp, &st) == -1) {
-			g_set_error(&task->err, rspamd_task_quark(), RSPAMD_PROTOCOL_ERROR,
-						"Invalid file (%s): %s", fp, strerror(errno));
-			return FALSE;
-		}
+		if (tok) {
+			debug_task("want to scan file %T", tok);
 
-		if (G_UNLIKELY(st.st_size == 0)) {
-			/* Empty file */
-			task->flags |= RSPAMD_TASK_FLAG_EMPTY;
-			task->msg.begin = rspamd_mempool_strdup(task->task_pool, "");
-			task->msg.len = 0;
-		}
-		else {
-			fd = open(fp, O_RDONLY);
+			r = rspamd_strlcpy(filepath, tok->begin,
+							   MIN(sizeof(filepath), tok->len + 1));
 
-			if (fd == -1) {
-				g_set_error(&task->err, rspamd_task_quark(),
-							RSPAMD_PROTOCOL_ERROR,
-							"Cannot open file (%s): %s", fp, strerror(errno));
+			rspamd_url_decode(filepath, filepath, r + 1);
+			flen = strlen(filepath);
+
+			if (filepath[0] == '"' && flen > 2) {
+				/* We need to unquote filepath */
+				fp = &filepath[1];
+				fp[flen - 2] = '\0';
+			}
+			else {
+				fp = &filepath[0];
+			}
+
+			if (stat(fp, &st) == -1) {
+				g_set_error(&task->err, rspamd_task_quark(), RSPAMD_PROTOCOL_ERROR,
+							"Invalid file (%s): %s", fp, strerror(errno));
 				return FALSE;
 			}
 
-			map = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
+			if (G_UNLIKELY(st.st_size == 0)) {
+				/* Empty file */
+				task->flags |= RSPAMD_TASK_FLAG_EMPTY;
+				task->msg.begin = rspamd_mempool_strdup(task->task_pool, "");
+				task->msg.len = 0;
+			}
+			else {
+				fd = open(fp, O_RDONLY);
+
+				if (fd == -1) {
+					g_set_error(&task->err, rspamd_task_quark(),
+								RSPAMD_PROTOCOL_ERROR,
+								"Cannot open file (%s): %s", fp, strerror(errno));
+					return FALSE;
+				}
+
+				map = mmap(NULL, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
 
 
-			if (map == MAP_FAILED) {
-				close(fd);
-				g_set_error(&task->err, rspamd_task_quark(),
-							RSPAMD_PROTOCOL_ERROR,
-							"Cannot mmap file (%s): %s", fp, strerror(errno));
-				return FALSE;
+				if (map == MAP_FAILED) {
+					close(fd);
+					g_set_error(&task->err, rspamd_task_quark(),
+								RSPAMD_PROTOCOL_ERROR,
+								"Cannot mmap file (%s): %s", fp, strerror(errno));
+					return FALSE;
+				}
+
+				task->msg.begin = map;
+				task->msg.len = st.st_size;
+				m = rspamd_mempool_alloc(task->task_pool, sizeof(*m));
+				m->begin = map;
+				m->len = st.st_size;
+				m->fd = fd;
+
+				rspamd_mempool_add_destructor(task->task_pool, rspamd_task_unmapper, m);
 			}
 
-			task->msg.begin = map;
-			task->msg.len = st.st_size;
-			m = rspamd_mempool_alloc(task->task_pool, sizeof(*m));
-			m->begin = map;
-			m->len = st.st_size;
-			m->fd = fd;
+			task->msg.fpath = rspamd_mempool_strdup(task->task_pool, fp);
+			task->flags |= RSPAMD_TASK_FLAG_FILE;
 
-			rspamd_mempool_add_destructor(task->task_pool, rspamd_task_unmapper, m);
+			msg_info_task("loaded message from file %s", fp);
+		}
+		else {
+			/* Plain data */
+			task->msg.begin = start;
+			task->msg.len = len;
 		}
-
-		task->msg.fpath = rspamd_mempool_strdup(task->task_pool, fp);
-		task->flags |= RSPAMD_TASK_FLAG_FILE;
-
-		msg_info_task("loaded message from file %s", fp);
-
-		return TRUE;
 	}
 
-	/* Plain data */
+
 	debug_task("got input of length %z", task->msg.len);
 
 	/* Check compression */
@@ -536,7 +535,7 @@ rspamd_task_load_message(struct rspamd_task *task,
 			ZSTD_DStream *zstream;
 			ZSTD_inBuffer zin;
 			ZSTD_outBuffer zout;
-			guchar *out;
+			unsigned char *out;
 			gsize outlen, r;
 			gulong dict_id;
 
@@ -577,10 +576,10 @@ rspamd_task_load_message(struct rspamd_task *task,
 			zstream = task->cfg->libs_ctx->in_zstream;
 
 			zin.pos = 0;
-			zin.src = start;
-			zin.size = len;
+			zin.src = task->msg.begin;
+			zin.size = task->msg.len;
 
-			if ((outlen = ZSTD_getDecompressedSize(start, len)) == 0) {
+			if ((outlen = ZSTD_getDecompressedSize(task->msg.begin, task->msg.len)) == 0) {
 				outlen = ZSTD_DStreamOutSize();
 			}
 
@@ -622,10 +621,6 @@ rspamd_task_load_message(struct rspamd_task *task,
 			return FALSE;
 		}
 	}
-	else {
-		task->msg.begin = start;
-		task->msg.len = len;
-	}
 
 	if (task->msg.len == 0) {
 		task->flags |= RSPAMD_TASK_FLAG_EMPTY;
@@ -669,10 +664,10 @@ rspamd_task_load_message(struct rspamd_task *task,
 	return TRUE;
 }
 
-static gint
-rspamd_task_select_processing_stage(struct rspamd_task *task, guint stages)
+static unsigned int
+rspamd_task_select_processing_stage(struct rspamd_task *task, unsigned int stages)
 {
-	gint st, mask;
+	unsigned int st, mask;
 
 	mask = task->processed_stages;
 
@@ -681,7 +676,7 @@ rspamd_task_select_processing_stage(struct rspamd_task *task, guint stages)
 	}
 	else {
 		for (st = 1; mask != 1; st++) {
-			mask = (unsigned int) mask >> 1;
+			mask = mask >> 1u;
 		}
 	}
 
@@ -701,9 +696,9 @@ rspamd_task_select_processing_stage(struct rspamd_task *task, guint stages)
 }
 
 gboolean
-rspamd_task_process(struct rspamd_task *task, guint stages)
+rspamd_task_process(struct rspamd_task *task, unsigned int stages)
 {
-	gint st;
+	unsigned int st;
 	gboolean ret = TRUE, all_done = TRUE;
 	GError *stat_error = NULL;
 
@@ -763,6 +758,10 @@ rspamd_task_process(struct rspamd_task *task, guint stages)
 		all_done = rspamd_symcache_process_symbols(task, task->cfg->cache,
 												   st);
 
+		if (all_done) {
+			rspamd_task_result_adjust_grow_factor(task, task->result, task->cfg->grow_factor);
+		}
+
 		if (all_done && (task->flags & RSPAMD_TASK_FLAG_LEARN_AUTO) &&
 			!RSPAMD_TASK_IS_EMPTY(task) &&
 			!(task->flags & (RSPAMD_TASK_FLAG_LEARN_SPAM | RSPAMD_TASK_FLAG_LEARN_HAM))) {
@@ -888,11 +887,11 @@ rspamd_task_get_sender(struct rspamd_task *task)
 	return task->from_envelope;
 }
 
-static const gchar *
+static const char *
 rspamd_task_cache_principal_recipient(struct rspamd_task *task,
-									  const gchar *rcpt, gsize len)
+									  const char *rcpt, gsize len)
 {
-	gchar *rcpt_lc;
+	char *rcpt_lc;
 
 	if (rcpt == NULL) {
 		return NULL;
@@ -908,12 +907,12 @@ rspamd_task_cache_principal_recipient(struct rspamd_task *task,
 	return rcpt_lc;
 }
 
-const gchar *
+const char *
 rspamd_task_get_principal_recipient(struct rspamd_task *task)
 {
-	const gchar *val;
+	const char *val;
 	struct rspamd_email_address *addr;
-	guint i;
+	unsigned int i;
 
 	val = rspamd_mempool_get_variable(task->task_pool,
 									  RSPAMD_MEMPOOL_PRINCIPAL_RECIPIENT);
@@ -954,9 +953,12 @@ rspamd_task_get_principal_recipient(struct rspamd_task *task)
 gboolean
 rspamd_learn_task_spam(struct rspamd_task *task,
 					   gboolean is_spam,
-					   const gchar *classifier,
+					   const char *classifier,
 					   GError **err)
 {
+	/* Disable learn auto flag to avoid bad learn codes */
+	task->flags &= ~RSPAMD_TASK_FLAG_LEARN_AUTO;
+
 	if (is_spam) {
 		task->flags |= RSPAMD_TASK_FLAG_LEARN_SPAM;
 	}
@@ -1047,12 +1049,12 @@ rspamd_task_log_check_condition(struct rspamd_task *task,
 /*
  * Sort by symbol's score -> name
  */
-static gint
+static int
 rspamd_task_compare_log_sym(gconstpointer a, gconstpointer b)
 {
 	const struct rspamd_symbol_result *s1 = *(const struct rspamd_symbol_result **) a,
 									  *s2 = *(const struct rspamd_symbol_result **) b;
-	gdouble w1, w2;
+	double w1, w2;
 
 
 	w1 = fabs(s1->score);
@@ -1065,7 +1067,7 @@ rspamd_task_compare_log_sym(gconstpointer a, gconstpointer b)
 	return (w2 - w1) * 1000.0;
 }
 
-static gint
+static int
 rspamd_task_compare_log_group(gconstpointer a, gconstpointer b)
 {
 	const struct rspamd_symbols_group *s1 = *(const struct rspamd_symbols_group **) a,
@@ -1079,7 +1081,7 @@ static rspamd_ftok_t
 rspamd_task_log_metric_res(struct rspamd_task *task,
 						   struct rspamd_log_format *lf)
 {
-	static gchar scorebuf[32];
+	static char scorebuf[32];
 	rspamd_ftok_t res = {.begin = NULL, .len = 0};
 	struct rspamd_scan_result *mres;
 	gboolean first = TRUE;
@@ -1088,8 +1090,9 @@ rspamd_task_log_metric_res(struct rspamd_task *task,
 	GPtrArray *sorted_symbols;
 	struct rspamd_action *act;
 	struct rspamd_symbols_group *gr;
-	guint i, j;
+	unsigned int i, j;
 	khiter_t k;
+	unsigned int max_log_elts = task->cfg->log_task_max_elts;
 
 	mres = task->result;
 	act = rspamd_check_action_metric(task, NULL, NULL);
@@ -1155,9 +1158,9 @@ rspamd_task_log_metric_res(struct rspamd_task *task,
 						DL_FOREACH(sym->opts_head, opt)
 						{
 							rspamd_printf_fstring(&symbuf, "%*s;",
-												  (gint) opt->optlen, opt->option);
+												  (int) opt->optlen, opt->option);
 
-							if (j >= max_log_elts) {
+							if (j >= max_log_elts && opt->next) {
 								rspamd_printf_fstring(&symbuf, "...;");
 								break;
 							}
@@ -1240,7 +1243,7 @@ rspamd_task_log_write_var(struct rspamd_task *task, rspamd_fstring_t *logbuf,
 						  const rspamd_ftok_t *var, const rspamd_ftok_t *content)
 {
 	rspamd_fstring_t *res = logbuf;
-	const gchar *p, *c, *end;
+	const char *p, *c, *end;
 
 	if (content == NULL) {
 		/* Just output variable */
@@ -1277,15 +1280,16 @@ rspamd_task_log_write_var(struct rspamd_task *task, rspamd_fstring_t *logbuf,
 
 static rspamd_fstring_t *
 rspamd_task_write_ialist(struct rspamd_task *task,
-						 GPtrArray *addrs, gint lim,
+						 GPtrArray *addrs, int lim,
 						 struct rspamd_log_format *lf,
 						 rspamd_fstring_t *logbuf)
 {
 	rspamd_fstring_t *res = logbuf, *varbuf;
 	rspamd_ftok_t var = {.begin = NULL, .len = 0};
 	struct rspamd_email_address *addr;
-	gint i, nchars = 0, wr = 0, cur_chars;
+	int i, nchars = 0, wr = 0, cur_chars;
 	gboolean has_orig = FALSE;
+	unsigned int max_log_elts = task->cfg->log_task_max_elts;
 
 	if (addrs && lim <= 0) {
 		lim = addrs->len;
@@ -1314,19 +1318,19 @@ rspamd_task_write_ialist(struct rspamd_task *task,
 			}
 		}
 
+		bool last = i == lim - 1;
+
 		cur_chars = addr->addr_len;
 		varbuf = rspamd_fstring_append(varbuf, addr->addr,
 									   cur_chars);
 		nchars += cur_chars;
 		wr++;
 
-		if (varbuf->len > 0) {
-			if (i != lim - 1) {
-				varbuf = rspamd_fstring_append(varbuf, ",", 1);
-			}
+		if (varbuf->len > 0 && !last) {
+			varbuf = rspamd_fstring_append(varbuf, ",", 1);
 		}
 
-		if (wr >= max_log_elts || nchars >= max_log_elts * 10) {
+		if (!last && (wr >= max_log_elts || nchars >= max_log_elts * 16)) {
 			varbuf = rspamd_fstring_append(varbuf, "...", 3);
 			break;
 		}
@@ -1346,14 +1350,15 @@ rspamd_task_write_ialist(struct rspamd_task *task,
 
 static rspamd_fstring_t *
 rspamd_task_write_addr_list(struct rspamd_task *task,
-							GPtrArray *addrs, gint lim,
+							GPtrArray *addrs, int lim,
 							struct rspamd_log_format *lf,
 							rspamd_fstring_t *logbuf)
 {
 	rspamd_fstring_t *res = logbuf, *varbuf;
 	rspamd_ftok_t var = {.begin = NULL, .len = 0};
 	struct rspamd_email_address *addr;
-	gint i;
+	unsigned int max_log_elts = task->cfg->log_task_max_elts;
+	unsigned int i;
 
 	if (lim <= 0) {
 		lim = addrs->len;
@@ -1363,18 +1368,17 @@ rspamd_task_write_addr_list(struct rspamd_task *task,
 
 	for (i = 0; i < lim; i++) {
 		addr = g_ptr_array_index(addrs, i);
+		bool last = i == lim - 1;
 
 		if (addr->addr) {
 			varbuf = rspamd_fstring_append(varbuf, addr->addr, addr->addr_len);
 		}
 
-		if (varbuf->len > 0) {
-			if (i != lim - 1) {
-				varbuf = rspamd_fstring_append(varbuf, ",", 1);
-			}
+		if (varbuf->len > 0 && !last) {
+			varbuf = rspamd_fstring_append(varbuf, ",", 1);
 		}
 
-		if (i >= max_log_elts) {
+		if (!last && i >= max_log_elts) {
 			varbuf = rspamd_fstring_append(varbuf, "...", 3);
 			break;
 		}
@@ -1398,8 +1402,8 @@ rspamd_task_log_variable(struct rspamd_task *task,
 {
 	rspamd_fstring_t *res = logbuf;
 	rspamd_ftok_t var = {.begin = NULL, .len = 0};
-	static gchar numbuf[128];
-	static const gchar undef[] = "undef";
+	static char numbuf[128];
+	static const char undef[] = "undef";
 
 	switch (lf->type) {
 	/* String vars */
@@ -1455,11 +1459,6 @@ rspamd_task_log_variable(struct rspamd_task *task,
 		var.begin = numbuf;
 		break;
 	case RSPAMD_LOG_TIME_REAL:
-		var.begin = rspamd_log_check_time(task->task_timestamp,
-										  task->time_real_finish,
-										  task->cfg->clock_res);
-		var.len = strlen(var.begin);
-		break;
 	case RSPAMD_LOG_TIME_VIRTUAL:
 		var.begin = rspamd_log_check_time(task->task_timestamp,
 										  task->time_real_finish,
@@ -1515,7 +1514,7 @@ rspamd_task_log_variable(struct rspamd_task *task,
 	case RSPAMD_LOG_DIGEST:
 		if (task->message) {
 			var.len = rspamd_snprintf(numbuf, sizeof(numbuf), "%*xs",
-									  (gint) sizeof(MESSAGE_FIELD(task, digest)),
+									  (int) sizeof(MESSAGE_FIELD(task, digest)),
 									  MESSAGE_FIELD(task, digest));
 			var.begin = numbuf;
 		}
@@ -1600,7 +1599,7 @@ void rspamd_task_write_log(struct rspamd_task *task)
 	rspamd_fstring_t *logbuf;
 	struct rspamd_log_format *lf;
 	struct rspamd_task **ptask;
-	const gchar *lua_str;
+	const char *lua_str;
 	gsize lua_str_len;
 	lua_State *L;
 
@@ -1624,7 +1623,7 @@ void rspamd_task_write_log(struct rspamd_task *task)
 			L = task->cfg->lua_state;
 			lua_rawgeti(L, LUA_REGISTRYINDEX, GPOINTER_TO_INT(lf->data));
 			ptask = lua_newuserdata(L, sizeof(*ptask));
-			rspamd_lua_setclass(L, "rspamd{task}", -1);
+			rspamd_lua_setclass(L, rspamd_task_classname, -1);
 			*ptask = task;
 
 			if (lua_pcall(L, 1, 1, 0) != 0) {
@@ -1659,11 +1658,9 @@ void rspamd_task_write_log(struct rspamd_task *task)
 	rspamd_fstring_free(logbuf);
 }
 
-gdouble
+double
 rspamd_task_get_required_score(struct rspamd_task *task, struct rspamd_scan_result *m)
 {
-	gint i;
-
 	if (m == NULL) {
 		m = task->result;
 
@@ -1672,7 +1669,7 @@ rspamd_task_get_required_score(struct rspamd_task *task, struct rspamd_scan_resu
 		}
 	}
 
-	for (i = m->nactions - 1; i >= 0; i--) {
+	for (unsigned int i = m->nactions; i-- > 0;) {
 		struct rspamd_action_config *action_lim = &m->actions_config[i];
 
 
@@ -1687,7 +1684,7 @@ rspamd_task_get_required_score(struct rspamd_task *task, struct rspamd_scan_resu
 
 rspamd_ftok_t *
 rspamd_task_get_request_header(struct rspamd_task *task,
-							   const gchar *name)
+							   const char *name)
 {
 	struct rspamd_request_header_chain *ret =
 		rspamd_task_get_request_header_multiple(task, name);
@@ -1701,13 +1698,13 @@ rspamd_task_get_request_header(struct rspamd_task *task,
 
 struct rspamd_request_header_chain *
 rspamd_task_get_request_header_multiple(struct rspamd_task *task,
-										const gchar *name)
+										const char *name)
 {
 	struct rspamd_request_header_chain *ret = NULL;
 	rspamd_ftok_t srch;
 	khiter_t k;
 
-	srch.begin = (gchar *) name;
+	srch.begin = (char *) name;
 	srch.len = strlen(name);
 
 	k = kh_get(rspamd_req_headers_hash, task->request_headers,
@@ -1726,7 +1723,7 @@ void rspamd_task_add_request_header(struct rspamd_task *task,
 {
 
 	khiter_t k;
-	gint res;
+	int res;
 	struct rspamd_request_header_chain *chain, *nchain;
 
 	k = kh_put(rspamd_req_headers_hash, task->request_headers,
@@ -1752,11 +1749,11 @@ void rspamd_task_add_request_header(struct rspamd_task *task,
 }
 
 
-void rspamd_task_profile_set(struct rspamd_task *task, const gchar *key,
-							 gdouble value)
+void rspamd_task_profile_set(struct rspamd_task *task, const char *key,
+							 double value)
 {
 	GHashTable *tbl;
-	gdouble *pval;
+	double *pval;
 
 	if (key == NULL) {
 		return;
@@ -1782,11 +1779,11 @@ void rspamd_task_profile_set(struct rspamd_task *task, const gchar *key,
 	}
 }
 
-gdouble *
-rspamd_task_profile_get(struct rspamd_task *task, const gchar *key)
+double *
+rspamd_task_profile_get(struct rspamd_task *task, const char *key)
 {
 	GHashTable *tbl;
-	gdouble *pval = NULL;
+	double *pval = NULL;
 
 	tbl = rspamd_mempool_get_variable(task->task_pool, RSPAMD_MEMPOOL_PROFILE);
 
@@ -1810,10 +1807,10 @@ rspamd_task_set_finish_time(struct rspamd_task *task)
 	return FALSE;
 }
 
-const gchar *
+const char *
 rspamd_task_stage_name(enum rspamd_task_stage stg)
 {
-	const gchar *ret = "unknown stage";
+	const char *ret = "unknown stage";
 
 	switch (stg) {
 	case RSPAMD_TASK_STAGE_CONNECT:
@@ -1946,7 +1943,7 @@ void rspamd_task_timeout(EV_P_ ev_timer *w, int revents)
 void rspamd_worker_guard_handler(EV_P_ ev_io *w, int revents)
 {
 	struct rspamd_task *task = (struct rspamd_task *) w->data;
-	gchar fake_buf[1024];
+	char fake_buf[1024];
 	gssize r;
 
 	r = read(w->fd, fake_buf, sizeof(fake_buf));
