@@ -24,11 +24,15 @@
 #include "backends/backends.h"
 #include "learn_cache/learn_cache.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct rspamd_statfile_runtime {
 	struct rspamd_statfile_config *st;
 	gpointer backend_runtime;
-	guint64 hits;
-	guint64 total_hits;
+	uint64_t hits;
+	uint64_t total_hits;
 };
 
 /* Common classifier structure */
@@ -39,12 +43,14 @@ struct rspamd_classifier {
 	gpointer cachecf;
 	gulong spam_learns;
 	gulong ham_learns;
+	int autolearn_cbref;
 	struct rspamd_classifier_config *cfg;
 	struct rspamd_stat_classifier *subrs;
+	gpointer specific;
 };
 
 struct rspamd_statfile {
-	gint id;
+	int id;
 	struct rspamd_statfile_config *stcf;
 	struct rspamd_classifier *classifier;
 	struct rspamd_stat_backend *backend;
@@ -54,16 +60,17 @@ struct rspamd_statfile {
 struct rspamd_stat_async_elt;
 
 typedef void (*rspamd_stat_async_handler)(struct rspamd_stat_async_elt *elt,
-		gpointer ud);
+										  gpointer ud);
+
 typedef void (*rspamd_stat_async_cleanup)(struct rspamd_stat_async_elt *elt,
-		gpointer ud);
+										  gpointer ud);
 
 struct rspamd_stat_async_elt {
 	rspamd_stat_async_handler handler;
 	rspamd_stat_async_cleanup cleanup;
-	struct event timer_ev;
-	struct timeval tv;
-	gdouble timeout;
+	struct ev_loop *event_loop;
+	ev_timer timer_ev;
+	double timeout;
 	gboolean enabled;
 	gpointer ud;
 	ref_entry_t ref;
@@ -72,44 +79,56 @@ struct rspamd_stat_async_elt {
 struct rspamd_stat_ctx {
 	/* Subroutines for all objects */
 	struct rspamd_stat_classifier *classifiers_subrs;
-	guint classifiers_count;
+	unsigned int classifiers_count;
 	struct rspamd_stat_tokenizer *tokenizers_subrs;
-	guint tokenizers_count;
+	unsigned int tokenizers_count;
 	struct rspamd_stat_backend *backends_subrs;
-	guint backends_count;
+	unsigned int backends_count;
 	struct rspamd_stat_cache *caches_subrs;
-	guint caches_count;
+	unsigned int caches_count;
 
 	/* Runtime configuration */
-	GPtrArray *statfiles; /* struct rspamd_statfile */
+	GPtrArray *statfiles;   /* struct rspamd_statfile */
 	GPtrArray *classifiers; /* struct rspamd_classifier */
-	GQueue *async_elts; /* struct rspamd_stat_async_elt */
+	GQueue *async_elts;     /* struct rspamd_stat_async_elt */
 	struct rspamd_config *cfg;
+
+	int lua_stat_tokens_ref;
+
 	/* Global tokenizer */
 	struct rspamd_stat_tokenizer *tokenizer;
 	gpointer tkcf;
 
-	struct event_base *ev_base;
+	struct ev_loop *event_loop;
 };
 
 typedef enum rspamd_learn_cache_result {
 	RSPAMD_LEARN_OK = 0,
 	RSPAMD_LEARN_UNLEARN,
-	RSPAMD_LEARN_INGORE
+	RSPAMD_LEARN_IGNORE
 } rspamd_learn_t;
 
-struct rspamd_stat_ctx * rspamd_stat_get_ctx (void);
-struct rspamd_stat_classifier * rspamd_stat_get_classifier (const gchar *name);
-struct rspamd_stat_backend * rspamd_stat_get_backend (const gchar *name);
-struct rspamd_stat_tokenizer * rspamd_stat_get_tokenizer (const gchar *name);
-struct rspamd_stat_cache * rspamd_stat_get_cache (const gchar *name);
-struct rspamd_stat_async_elt* rspamd_stat_ctx_register_async (
-		rspamd_stat_async_handler handler, rspamd_stat_async_cleanup cleanup,
-		gpointer d, gdouble timeout);
+struct rspamd_stat_ctx *rspamd_stat_get_ctx(void);
 
-static GQuark rspamd_stat_quark (void)
+struct rspamd_stat_classifier *rspamd_stat_get_classifier(const char *name);
+
+struct rspamd_stat_backend *rspamd_stat_get_backend(const char *name);
+
+struct rspamd_stat_tokenizer *rspamd_stat_get_tokenizer(const char *name);
+
+struct rspamd_stat_cache *rspamd_stat_get_cache(const char *name);
+
+struct rspamd_stat_async_elt *rspamd_stat_ctx_register_async(
+	rspamd_stat_async_handler handler, rspamd_stat_async_cleanup cleanup,
+	gpointer d, double timeout);
+
+static GQuark rspamd_stat_quark(void)
 {
-	return g_quark_from_static_string ("rspamd-statistics");
+	return g_quark_from_static_string("rspamd-statistics");
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* STAT_INTERNAL_H_ */
